@@ -8,7 +8,7 @@ from enum import IntEnum
 from typing import Optional, Set, List, Dict, Iterator, Type
 from logging import Logger
 
-from pydantic import BaseModel, ByteSize, Field, ValidationError
+from pydantic import BaseModel, ByteSize, Field, ValidationError, parse_obj_as
 
 from .exception import PipelineError, PipelineInputError, PipelineOutputError
 from .message import Message, DescribeMessage
@@ -120,6 +120,8 @@ class Worker(ABC):
 
 
 class ProducerSettings(WorkerSettings):
+    """Producer settings"""
+
     pass
 
 
@@ -127,16 +129,27 @@ class Producer(Worker):
     """Producer is a worker to generate new messages. For example, a webcrawler can
     be a producer. It reads no input, and produce outputs until it exits.
 
+    Parameters:
+        :param settings: settings
+        :type settings: ProducerSettings
+        :param output_class: output class
+        :type output_class: Type[BaseModel]
+        :param logger: logger
+        :type logger: Logger
+
     Usage:
-    >>> from pydantic import BaseModel
-    >>>
-    >>> class Output(BaseModel):
-    ...     pass
-    >>>
-    >>> settings = ProducerSettings(name='', version='', description='', out_kind='MEM')
-    >>> producer = Producer(settings, output_class=Output)
-    >>> producer.parse_args()
-    >>> #producer.start()
+
+    .. code-block:: python
+
+        >>> from pydantic import BaseModel
+        >>>
+        >>> class Output(BaseModel):
+        ...     pass
+        >>>
+        >>> settings = ProducerSettings(name='', version='', description='', out_kind='MEM')
+        >>> producer = Producer(settings, output_class=Output)
+        >>> producer.parse_args()
+        >>> #producer.start()
     """
 
     generator: Iterator[BaseModel]
@@ -193,7 +206,7 @@ class Producer(Worker):
                 output = self._step()
                 self.timer.log(self.logger)
                 if hasattr(output, "id"):
-                    msg = Message(id=output.id, content=output.dict())
+                    msg = Message(id=output.id, content=output.dict())  # type: ignore
                 else:
                     msg = Message(content=output.dict())
                 self.logger.info("Generated %d-th message %s", i, msg)
@@ -315,6 +328,11 @@ class ProcessorSettings(WorkerSettings):
 
 
 class Processor(Worker):
+    """Processor is a worker which will process incoming messages and output
+    new messages
+
+    """
+
     settings: ProcessorSettings
     input_class: Type[BaseModel]
     output_class: Type[BaseModel]
@@ -333,22 +351,9 @@ class Processor(Worker):
         self.output_class = output_class
         self.destination_class = None
 
-    def use_retry_topic(self, name: str = None) -> None:
-        """Retry topic is introduced to solve error handling by saving
-        message being processed when error occurs to a separate topic.
-        In this way, these messages can be reprocessed at a later stage.
-        """
-        if self.destination_class:
-            settings = copy(self.destination.settings)
-            settings.topic = name if name else self.name + "-retry"
-            self.retryDestination = self.destination_class(
-                settings=settings, logger=self.logger
-            )
-            self.retryEnabled = True
-        else:
-            raise PipelineError("Please call this function after parse_args")
-
-    def process(self, msg: BaseModel) -> BaseModel:
+    def process(
+        self, message_content: BaseModel, message_id: Optional[str] = None
+    ) -> BaseModel:
         """process function to be overridden by users, for streaming
         processing, this function needs to do in-place update on msg.dct
         and return an error or a list of errors (for batch processing).
@@ -366,7 +371,7 @@ class Processor(Worker):
         raise NotImplementedError("You need to implement .process()")
 
     def _step(self, msg: Message) -> None:
-        """ process one message """
+        """process one message"""
 
         self.logger.info(f"Receive message {msg}")
         log = Log(
@@ -383,11 +388,11 @@ class Processor(Worker):
             self.logger.error(e.json())
             raise PipelineInputError(f"Input validation failed for message {msg}")
 
-        output_data = self.process(input_data)
+        output_data = self.process(input_data, msg.id)
         self.logger.info(f"Processed message {msg}")
 
         try:
-            output_model = self.output_class(**output_data.dict())
+            output_model = parse_obj_as(self.output_class, output_data)
             self.logger.info(f"Validated output {output_model}")
         except ValidationError as e:
             self.logger.error(f"Output validation failed for message {msg}")
@@ -429,7 +434,7 @@ class Processor(Worker):
             self.monitor.record_write(self.destination.topic)
 
     def start(self) -> None:
-        """ start processing. """
+        """start processing."""
 
         self.logger.setLevel(level=logging.INFO)
         if self.settings.debug:
